@@ -1,15 +1,18 @@
+from bossman.plugins.akamai.lib.edgegrid import EdgegridError
 import re
 import time
 import json
 import random
+import pathlib
 from io import StringIO
 from os import getenv
 from os.path import expanduser, basename, dirname, join
 from collections import OrderedDict
+from types import SimpleNamespace
 import jsonschema
 
 from bossman.cache import cache
-from bossman.errors import BossmanError
+from bossman.errors import BossmanError, BossmanValidationError
 from bossman.abc import ResourceTypeABC
 from bossman.abc import ResourceStatusABC
 from bossman.abc import ResourceABC
@@ -29,7 +32,7 @@ RE_COMMIT = re.compile("^commit: ([a-z0-9]*)", re.MULTILINE)
 
 _cache = cache.key(__name__)
 
-class PropertyError(BossmanError):
+class PropertyValidationError(BossmanValidationError):
   pass
 
 class PropertyNotFoundError(BossmanError):
@@ -46,11 +49,13 @@ class PropertyResource(ResourceABC):
 
   @property
   def rules_path(self):
-    return join(self.path, "rules.json")
+    # All operations use unix-style paths; this is important
+    return str(pathlib.PurePosixPath(self.path) / "rules.json")
 
   @property
   def hostnames_path(self):
-    return join(self.path, "hostnames.json")
+    # All operations use unix-style paths; this is important
+    return str(pathlib.PurePosixPath(self.path) / "hostnames.json")
 
   @property
   def paths(self):
@@ -330,7 +335,7 @@ class ResourceType(ResourceTypeABC):
       # Get the rules json from the commit
       rules = revision.show_path(resource.rules_path)
       if rules is None:
-        raise PropertyError("missing rule tree")
+        raise PropertyValidationError("missing rule tree")
 
       # before changing anything in PAPI, check that the json files are valid
       rules_json = self.validate_rules(resource, rules)
@@ -341,6 +346,8 @@ class ResourceType(ResourceTypeABC):
       hostnames_json = None
       hostnames = revision.show_path(resource.hostnames_path)
       hostnames_json = self.validate_hostnames(resource, hostnames)
+    except EdgegridError as e:
+      raise e
     except RuntimeError as e:
       return PropertyApplyResult(resource, revision, error=e)
 
@@ -374,23 +381,31 @@ class ResourceType(ResourceTypeABC):
     except RuntimeError as e:
       return PropertyApplyResult(resource, revision, error=e)
 
+    notes = SimpleNamespace(
+      property_id=property_id,
+      property_version=next_version.propertyVersion,
+      etag=None,
+      has_errors=False
+    )
+
     try:
       # first, we apply the hostnames change
       hostnames_result = self.papi.update_property_hostnames(property_id, next_version.propertyVersion, hostnames_json)
+      notes.etag = hostnames_result.etag
+      notes.has_errors = notes.has_errors or hostnames_result.has_errors
       # then, regardless of whether there was a change to the rule tree, we need to update it with
       # the commit message and id, otherwise we will get a dirty status
       rule_tree_result = self.papi.update_property_rule_tree(property_id, next_version.propertyVersion, rules_json)
+      notes.etag = rule_tree_result.etag
+      notes.has_errors = notes.has_errors or rule_tree_result.has_errors
 
-      # Finally, assign the updated values to the git notes
-      revision.get_notes(resource.path).set(
-        property_version=rule_tree_result.propertyVersion,
-        property_id=rule_tree_result.propertyId,
-        etag=rule_tree_result.etag,
-        has_errors=rule_tree_result.has_errors or hostnames_result.has_errors
-      )
       return PropertyApplyResult(resource, revision, next_version, rule_tree=rule_tree_result, hostnames=hostnames_result)
     except RuntimeError as e:
+      notes.has_errors = True
       return PropertyApplyResult(resource, revision, error=e)
+    finally:
+      # Finally, assign the updated values to the git notes
+      revision.get_notes(resource.path).set(**vars(notes))
 
 
   def get_property_version_for_revision_id(self, property_id, revision_id):
@@ -420,12 +435,15 @@ class ResourceType(ResourceTypeABC):
     return None
 
   def validate_working_tree(self, resource: PropertyResource):
-    with open(resource.hostnames_path, "r") as hfd:
-      hostnames = hfd.read()
-      self.validate_hostnames(resource, hostnames)
-    with open(resource.rules_path, "r") as hfd:
-      rules = hfd.read()
-      self.validate_rules(resource, rules)
+    try:
+      with open(resource.hostnames_path, "r") as hfd:
+        hostnames = hfd.read()
+        self.validate_hostnames(resource, hostnames)
+      with open(resource.rules_path, "r") as hfd:
+        rules = hfd.read()
+        self.validate_rules(resource, rules)
+    except FileNotFoundError as e:
+      raise PropertyValidationError("file not found {}".format(e.filename))
 
   def validate_rules(self, resource: PropertyResource, rules: str):
     try:
@@ -436,17 +454,15 @@ class ResourceType(ResourceTypeABC):
       requiredFields = ("groupId", "contractId", "ruleFormat", "productId")
       missing = list(field for field in requiredFields if field not in rules_json)
       if len(missing):
-        raise PropertyError("{}: {} field is required".format(resource.rules_path, ", ".join(missing)))
+        raise PropertyValidationError("{}: {} field is required".format(resource.rules_path, ", ".join(missing)))
 
       schema = self.papi.get_rule_format_schema(rules_json.get("productId"), rules_json.get("ruleFormat"))
       jsonschema.validate(rules_json, schema)
       return rules_json
     except json.decoder.JSONDecodeError as err:
-      self.logger.error("{}: bad rules.json - {}", resource, err.args)
-      raise PropertyError(resource, "bad rules.json", err.args)
+      raise PropertyValidationError("bad rules.json", err.args)
     except jsonschema.ValidationError as err:
-      self.logger.error("{}: invalid rules.json - {}", resource, err.args)
-      raise PropertyError(resource, "invalid rules.json", err.args)
+      raise PropertyValidationError("invalid rules.json", err.message)
 
   def validate_hostnames(self, resource: PropertyResource, hostnames: str):
     try:
@@ -455,11 +471,9 @@ class ResourceType(ResourceTypeABC):
       jsonschema.validate(hostnames_json, schema)
       return hostnames_json
     except json.decoder.JSONDecodeError as err:
-      self.logger.error("{}: bad hostnames.json - {}", resource, err.args)
-      raise PropertyError(resource, "bad hostnames.json", err.args)
+      raise PropertyValidationError("bad hostnames.json", err.args)
     except jsonschema.ValidationError as err:
-      self.logger.error("{}: bad hostnames.json - {}", resource, err.args)
-      raise PropertyError(resource, "invalid hostnames.json", err.args)
+      raise PropertyValidationError("invalid hostnames.json", err.args)
 
   def _release(self, network: str, resource: ResourceABC, revision: Revision, on_update: callable = lambda resource, status, progress: None):
     notes = revision.get_notes(resource.path)
